# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"ClassificationInterpretationEx": "01_interpret_classification-interpretation.ipynb",
         "intersection": "02_interpret_compare-models.ipynb",
         "compare_venn": "02_interpret_compare-models.ipynb",
         "Hook": "03_interpret_gradcam.ipynb",
         "HookBwd": "03_interpret_gradcam.ipynb",
         "create_test_img": "03_interpret_gradcam.ipynb",
         "compute_gcam_items": "03_interpret_gradcam.ipynb",
         "compute_gcam_map": "03_interpret_gradcam.ipynb",
         "plt_decoded": "03_interpret_gradcam.ipynb",
         "plot_gcam": "03_interpret_gradcam.ipynb",
         "GradCam": "03_interpret_gradcam.ipynb",
         "PathLike": "04_inference_export-to-other-frameworks.ipynb",
         "to_cuda": "04_inference_export-to-other-frameworks.ipynb",
         "preprocess_one": "04_inference_export-to-other-frameworks.ipynb",
         "preprocess_batch": "04_inference_export-to-other-frameworks.ipynb",
         "open_image": "04_inference_export-to-other-frameworks.ipynb",
         "torch_to_onnx": "04_inference_export-to-other-frameworks.ipynb",
         "torch_to_numpy": "04_inference_export-to-other-frameworks.ipynb",
         "onnx_to_tf": "04_inference_export-to-other-frameworks.ipynb",
         "get_nn_spec": "04_inference_export-to-other-frameworks.ipynb",
         "onnx_to_coreml": "04_inference_export-to-other-frameworks.ipynb",
         "is_3dlut_row": "05_augment_PIL-img_filters.ipynb",
         "read_lut": "05_augment_PIL-img_filters.ipynb",
         "ApplyPILFilter": "05_augment_PIL-img_filters.ipynb",
         "albu_augment": "06_augment_albumentations.ipynb",
         "AlbumentationsWrapper": "06_augment_albumentations.ipynb",
         "BlurringTfms": "06_augment_albumentations.ipynb",
         "StyleTfms": "06_augment_albumentations.ipynb",
         "WeatherTfms": "06_augment_albumentations.ipynb",
         "NoiseTfms": "06_augment_albumentations.ipynb",
         "ColorTonesTfms": "06_augment_albumentations.ipynb",
         "ColorChannelTfms": "06_augment_albumentations.ipynb",
         "LightingTfms": "06_augment_albumentations.ipynb",
         "OtherTfms": "06_augment_albumentations.ipynb",
         "Tfms": "06_augment_albumentations.ipynb",
         "KorniaBase": "07_augment_kornia.ipynb"}

modules = ["interpret/interpret.py",
           "interpret/compare.py",
           "interpret/gradcam.py",
           "inference/export.py",
           "augment/pil_filters.py",
           "augment/albumentations.py",
           "augment/kornia.py"]

doc_url = "https://Synopsis.github.io/fastai2_extensions/"

git_url = "https://github.com/Synopsis/fastai2_extensions/tree/master/"

def custom_doc_links(name): return None
